<?php
declare(strict_types=1);

namespace Catequil\Invoice\Service;

use Catequil\Barcode\Resolver\ResolverInterface;
use Catequil\Invoice\Model\Invoice;
use Catequil\Invoice\Repository\InvoiceRepository;
use Doctrine\ORM\EntityManagerInterface;
use function intval;

/**
 * Class InvoiceBarcodeResolver
 *
 * @package Catequil\Invoice\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-invoice
 */
final class InvoiceBarcodeResolver implements ResolverInterface
{
    private EntityManagerInterface $em;

    /**
     * @param EntityManagerInterface $em
     */
    public function __construct(EntityManagerInterface $em)
    {
        $this->em = $em;
    }

    public function combine(int|string $value): ?string
    {
        /** @var InvoiceRepository $repo */
        $repo = $this->em->getRepository(Invoice::class);
        $invoice = $repo->find($value);

        return $invoice ? $invoice->getDocumentType().$invoice->getInvoiceNo() : null;
    }

    public function resolve(string $barcode): null|int|string
    {
        if (!preg_match('/^([A-Z]+)(\d+)$/', $barcode, $matches)) {
            return null;
        }

        /** @var InvoiceRepository $repo */
        $repo = $this->em->getRepository(Invoice::class);
        $invoice = $repo->findByInvoiceNo($matches[1], intval($matches[2]));

        return $invoice?->getId();
    }
}